#!/usr/bin/env bash
#
# _build/oil-native.sh - generated by ./NINJA_config.py
#
# Usage
#   _build/oil-native COMPILER? VARIANT? SKIP_REBUILD?
#
#   COMPILER: 'cxx' for system compiler, or 'clang' [default cxx]
#   VARIANT: 'dbg' or 'opt' [default dbg]
#   SKIP_REBUILD: if non-empty, checks if the output exists before building
#
# Could run with /bin/sh, but use bash for now, bceause dash has bad errors messages!
#!/bin/sh

. cpp/NINJA-steps.sh

main() {
  ### Compile oil-native into _bin/$compiler-$variant-sh/ (not with ninja)

  local compiler=${1:-cxx}   # default is system compiler
  local variant=${2:-opt}    # default is optimized build
  local skip_rebuild=${3:-}  # if the output exists, skip build'

  local more_cxx_flags='-D DUMB_ALLOC -D NO_GC_HACK -D LEAKY_BINDINGS'

  local out=_bin/$compiler-$variant-sh/osh_eval
  if test -n "$skip_rebuild" && test -f "$out"; then
    echo
    echo "$0: SKIPPING build because $out exists"
    echo
    return
  fi

  echo
  echo "$0: Building oil-native: $out"
  echo

  mkdir -p "_build/obj/$compiler-$variant-sh" "_bin/$compiler-$variant-sh"

  echo 'CXX cpp/leaky_core.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_core.cc "_build/obj/$compiler-$variant-sh/leaky_core.o"
  echo 'CXX cpp/leaky_frontend_flag_spec.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_frontend_flag_spec.cc "_build/obj/$compiler-$variant-sh/leaky_frontend_flag_spec.o"
  echo 'CXX cpp/leaky_frontend_match.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_frontend_match.cc "_build/obj/$compiler-$variant-sh/leaky_frontend_match.o"
  echo 'CXX cpp/leaky_frontend_tdop.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_frontend_tdop.cc "_build/obj/$compiler-$variant-sh/leaky_frontend_tdop.o"
  echo 'CXX cpp/leaky_osh.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_osh.cc "_build/obj/$compiler-$variant-sh/leaky_osh.o"
  echo 'CXX cpp/leaky_pgen2.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_pgen2.cc "_build/obj/$compiler-$variant-sh/leaky_pgen2.o"
  echo 'CXX cpp/leaky_pylib.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_pylib.cc "_build/obj/$compiler-$variant-sh/leaky_pylib.o"
  echo 'CXX cpp/leaky_dumb_alloc.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_dumb_alloc.cc "_build/obj/$compiler-$variant-sh/leaky_dumb_alloc.o"
  echo 'CXX cpp/leaky_stdlib.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_stdlib.cc "_build/obj/$compiler-$variant-sh/leaky_stdlib.o"
  echo 'CXX cpp/leaky_libc.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    cpp/leaky_libc.cc "_build/obj/$compiler-$variant-sh/leaky_libc.o"
  echo 'CXX _build/cpp/arg_types.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/arg_types.cc "_build/obj/$compiler-$variant-sh/arg_types.o"
  echo 'CXX _build/cpp/arith_parse.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/arith_parse.cc "_build/obj/$compiler-$variant-sh/arith_parse.o"
  echo 'CXX _build/cpp/consts.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/consts.cc "_build/obj/$compiler-$variant-sh/consts.o"
  echo 'CXX _build/cpp/osh_eval.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/osh_eval.cc "_build/obj/$compiler-$variant-sh/osh_eval.o"
  echo 'CXX _build/cpp/runtime_asdl.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/runtime_asdl.cc "_build/obj/$compiler-$variant-sh/runtime_asdl.o"
  echo 'CXX _build/cpp/syntax_asdl.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/syntax_asdl.cc "_build/obj/$compiler-$variant-sh/syntax_asdl.o"
  echo 'CXX _build/cpp/hnode_asdl.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/hnode_asdl.cc "_build/obj/$compiler-$variant-sh/hnode_asdl.o"
  echo 'CXX _build/cpp/id_kind_asdl.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    _build/cpp/id_kind_asdl.cc "_build/obj/$compiler-$variant-sh/id_kind_asdl.o"
  echo 'CXX mycpp/gc_heap.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    mycpp/gc_heap.cc "_build/obj/$compiler-$variant-sh/gc_heap.o"
  echo 'CXX mycpp/leaky_types.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    mycpp/leaky_types.cc "_build/obj/$compiler-$variant-sh/leaky_types.o"
  echo 'CXX mycpp/mylib_old.cc'
  compile_one "$compiler" "$variant" "$more_cxx_flags" \
    mycpp/mylib_old.cc "_build/obj/$compiler-$variant-sh/mylib_old.o"

  echo "LINK $out"
  link "$compiler" "$variant" "$out" \
    "_build/obj/$compiler-$variant-sh/leaky_core.o" \
    "_build/obj/$compiler-$variant-sh/leaky_frontend_flag_spec.o" \
    "_build/obj/$compiler-$variant-sh/leaky_frontend_match.o" \
    "_build/obj/$compiler-$variant-sh/leaky_frontend_tdop.o" \
    "_build/obj/$compiler-$variant-sh/leaky_osh.o" \
    "_build/obj/$compiler-$variant-sh/leaky_pgen2.o" \
    "_build/obj/$compiler-$variant-sh/leaky_pylib.o" \
    "_build/obj/$compiler-$variant-sh/leaky_dumb_alloc.o" \
    "_build/obj/$compiler-$variant-sh/leaky_stdlib.o" \
    "_build/obj/$compiler-$variant-sh/leaky_libc.o" \
    "_build/obj/$compiler-$variant-sh/arg_types.o" \
    "_build/obj/$compiler-$variant-sh/arith_parse.o" \
    "_build/obj/$compiler-$variant-sh/consts.o" \
    "_build/obj/$compiler-$variant-sh/osh_eval.o" \
    "_build/obj/$compiler-$variant-sh/runtime_asdl.o" \
    "_build/obj/$compiler-$variant-sh/syntax_asdl.o" \
    "_build/obj/$compiler-$variant-sh/hnode_asdl.o" \
    "_build/obj/$compiler-$variant-sh/id_kind_asdl.o" \
    "_build/obj/$compiler-$variant-sh/gc_heap.o" \
    "_build/obj/$compiler-$variant-sh/leaky_types.o" \
    "_build/obj/$compiler-$variant-sh/mylib_old.o"

  if test "$variant" = opt; then
    strip -o "$out.stripped" "$out"
  fi
}

main "$@"

